@@ -5,13 +5,15 @@ import json
 import logging
 import aiofiles
 import numpy as np
-from typing import Dict, Set, Optional, List, Union
+from typing import Dict, Set, Optional, List, Union, cast
 from functools import partial
+import os
 
 from medcat import __version__
 from medcat.utils.hasher import Hasher
 from medcat.utils.matutils import unitvec
 from medcat.utils.ml_utils import get_lr_linking
+from medcat.utils.decorators import deprecated
 from medcat.config import Config, weighted_average, workers
 from medcat.utils.saving.serializer import CDBSerializer
 
@@ -54,13 +56,17 @@ class CDB(object):
             for the base NER+L use-case, but can be useufl for Debugging or some special stuff.
         vocab (Dict[str, int]):
             Stores all the words tha appear in this CDB and the count for each one.
+        is_dirty (bool):
+            Whether or not the CDB has been changed since it was loaded or created
     """
 
     def __init__(self, config: Union[Config, None] = None) -> None:
         if config is None:
             self.config = Config()
+            self._config_from_file = False
         else:
             self.config = config
+            self._config_from_file = True
         self.name2cuis: Dict = {}
         self.name2cuis2status: Dict = {}
 
@@ -91,10 +97,19 @@ class CDB(object):
                 }
         self.vocab: Dict = {} # Vocabulary of all words ever in our cdb
         self._optim_params = None
+        self.is_dirty = False
+        self._hash: Optional[str] = None
+        # the config hash is kept track of here so that
+        # the CDB hash can be re-calculated when the config changes
+        # it can also be used to make sure the config loaded with
+        # a CDB matches the config it was saved with
+        # since the config is now saved separately
+        self._config_hash: Optional[str] = None
+        self._memory_optimised_parts: Set[str] = set()
 
     def get_name(self, cui: str) -> str:
         """Returns preferred name if it exists, otherwise it will return
-        the logest name assigend to the concept.
+        the longest name assigned to the concept.
 
         Args:
             cui
@@ -111,9 +126,10 @@ class CDB(object):
     def update_cui2average_confidence(self, cui: str, new_sim: float) -> None:
         self.cui2average_confidence[cui] = (self.cui2average_confidence.get(cui, 0) * self.cui2count_train.get(cui, 0) + new_sim) / \
                                             (self.cui2count_train.get(cui, 0) + 1)
+        self.is_dirty = True
 
     def remove_names(self, cui: str, names: Dict) -> None:
-        """Remove names from an existing concept - efect is this name will never again be used to link to this concept.
+        """Remove names from an existing concept - effect is this name will never again be used to link to this concept.
         This will only remove the name from the linker (namely name2cuis and name2cuis2status), the name will still be present everywhere else.
         Why? Because it is bothersome to remove it from everywhere, but
         could also be useful to keep the removed names in e.g. cui2names.
@@ -146,6 +162,44 @@ class CDB(object):
                             self.name2cuis2status[name][_cui] = 'N'
                         elif self.name2cuis2status[name][_cui] == 'P':
                             self.name2cuis2status[name][_cui] = 'PD'
+        self.is_dirty = True
+
+    def remove_cui(self, cui: str) -> None:
+        """This function takes a `CUI` as an argument and removes it from all the internal objects that reference it.
+        Args:
+            cui
+        """
+        if cui in self.cui2names:
+            del self.cui2names[cui]
+        if cui in self.cui2snames:
+            del self.cui2snames[cui]
+        if cui in self.cui2context_vectors:
+            del self.cui2context_vectors[cui]
+        if cui in self.cui2count_train:
+            del self.cui2count_train[cui]
+        if cui in self.cui2tags:
+            del self.cui2tags[cui]
+        if cui in self.cui2type_ids:
+            del self.cui2type_ids[cui]
+        if cui in self.cui2preferred_name:
+            del self.cui2preferred_name[cui]
+        if cui in self.cui2average_confidence:
+            del self.cui2average_confidence[cui]
+        for name, cuis in self.name2cuis.items():
+            if cui in cuis:
+                cuis.remove(cui)
+        for name, cuis2status in self.name2cuis2status.items():
+            if cui in cuis2status:
+                del cuis2status[cui]
+        if isinstance(self.snames, set):
+            # if this is a memory optimised CDB, this won't be a set
+            # but it also won't need to be changed since it
+            # relies directly on cui2snames
+            self.snames = set()
+            for cuis in self.cui2snames.values():
+                self.snames |= cuis
+        self.name2count_train = {name: len(cuis) for name, cuis in self.name2cuis.items()}
+        self.is_dirty = True
 
     def add_names(self, cui: str, names: Dict, name_status: str = 'A', full_build: bool = False) -> None:
         """Adds a name to an existing concept.
@@ -169,8 +223,9 @@ class CDB(object):
             # Name status must be one of the three
             name_status = 'A'
 
-        self.add_concept(cui=cui, names=names, ontologies=set(), name_status=name_status, type_ids=set(), description='', full_build=full_build)
+        self._add_concept(cui=cui, names=names, ontologies=set(), name_status=name_status, type_ids=set(), description='', full_build=full_build)
 
+    @deprecated("Use `cdb._add_concept` as this will be removed in a future release.")
     def add_concept(self,
                     cui: str,
                     names: Dict,
@@ -179,6 +234,43 @@ class CDB(object):
                     type_ids: Set[str],
                     description: str,
                     full_build: bool = False) -> None:
+        """
+        Deprecated: Use `cdb._add_concept` as this will be removed in a future release.
+
+        Add a concept to internal Concept Database (CDB). Depending on what you are providing
+        this will add a large number of properties for each concept.
+
+        Args:
+            cui (str):
+                Concept ID or unique identifier in this database, all concepts that have
+                the same CUI will be merged internally.
+            names (Dict[str, Dict]):
+                Names for this concept, or the value that if found in free text can be linked to this concept.
+                Names is a dict like: `{name: {'tokens': tokens, 'snames': snames, 'raw_name': raw_name}, ...}`
+                Names should be generated by helper function 'medcat.preprocessing.cleaners.prepare_name'
+            ontologies (Set[str]):
+                ontologies in which the concept exists (e.g. SNOMEDCT, HPO)
+            name_status (str):
+                One of `P`, `N`, `A`
+            type_ids (Set[str]):
+                Semantic type identifier (have a look at TUIs in UMLS or SNOMED-CT)
+            description (str):
+                Description of this concept.
+            full_build (bool):
+                If True the dictionary self.addl_info will also be populated, contains a lot of extra information
+                about concepts, but can be very memory consuming. This is not necessary
+                for normal functioning of MedCAT (Default Value `False`).
+        """
+        self._add_concept(cui, names, ontologies, name_status, type_ids, description, full_build)
+
+    def _add_concept(self,
+                    cui: str,
+                    names: Dict,
+                    ontologies: set,
+                    name_status: str,
+                    type_ids: Set[str],
+                    description: str,
+                    full_build: bool = False) -> None:
         """Add a concept to internal Concept Database (CDB). Depending on what you are providing
         this will add a large number of properties for each concept.
 
@@ -188,7 +280,8 @@ class CDB(object):
                 the same CUI will be merged internally.
             names (Dict[str, Dict]):
                 Names for this concept, or the value that if found in free text can be linked to this concept.
-                Names is an dict like: `{name: {'tokens': tokens, 'snames': snames, 'raw_name': raw_name}, ...}`
+                Names is a dict like: `{name: {'tokens': tokens, 'snames': snames, 'raw_name': raw_name}, ...}`
+                Names should be generated by helper function 'medcat.preprocessing.cleaners.prepare_name'
             ontologies (Set[str]):
                 ontologies in which the concept exists (e.g. SNOMEDCT, HPO)
             name_status (str):
@@ -265,6 +358,21 @@ class CDB(object):
             if name_status == 'P' and cui not in self.cui2preferred_name:
                 # Do not overwrite old preferred names
                 self.cui2preferred_name[cui] = name_info['raw_name']
+        elif names:
+            # if no name_info and names is NOT an empty dict
+            # this shouldn't really happen in the current setup
+            raise ValueError("Unknown state where there is no name_info, "
+                             "yet the `names` dict is not empty (%s)", names)
+        elif name_status == 'P' and cui not in self.cui2preferred_name:
+            # this means names is an empty `names` dict
+            logger.warning("Did not manage to add a preferred name in `add_cui`. "
+                           "Was trying to do so for cui: '%s'"
+                           "This means that the `names` dict passed was empty. "
+                           "This is _usually_ caused by either no name or too short "
+                           "a name passed to the `prepare_name` method. "
+                           "The minimum length is defined in: "
+                           "'config.cdb_maker.min_letters_required' and "
+                           "is currently set at %s", cui, self.config.cdb_maker['min_letters_required'])
 
         # Add other fields if full_build
         if full_build:
@@ -289,6 +397,7 @@ class CDB(object):
                     self.addl_info['type_id2cuis'][type_id].add(cui)
                 else:
                     self.addl_info['type_id2cuis'][type_id] = {cui}
+        self.is_dirty = True
 
     def add_addl_info(self, name: str, data: Dict, reset_existing: bool = False) -> None:
         """Add data to the addl_info dictionary. This is done in a function to
@@ -306,6 +415,7 @@ class CDB(object):
             self.addl_info[name] = {}
 
         self.addl_info[name].update(data)
+        self.is_dirty = True
 
     def update_context_vector(self,
                               cui: str,
@@ -371,8 +481,10 @@ class CDB(object):
         if not negative:
             # Increase counter only for positive examples
             self.cui2count_train[cui] += 1
+        self.is_dirty = True
 
-    def save(self, path: str, json_path: Optional[str] = None, overwrite: bool = True) -> None:
+    def save(self, path: str, json_path: Optional[str] = None, overwrite: bool = True,
+            calc_hash_if_missing: bool = False) -> None:
         """Saves model to file (in fact it saves variables of this class).
 
         If a `json_path` is specified, the JSON serialization is used for some of the data.
@@ -384,7 +496,12 @@ class CDB(object):
                 If specified, json serialisation is used. Defaults to None.
             overwrite (bool):
                 Whether or not to overwrite existing file(s).
+            calc_hash_if_missing (bool):
+                Calculate the hash if it's missing. Defaults to `False`
         """
+        if calc_hash_if_missing and not self._hash:
+            # get instead of calculate so that the CDB is marked as not dirty if it was dirty
+            self.get_hash()
         ser = CDBSerializer(path, json_path)
         ser.serialize(self, overwrite=overwrite)
 
@@ -405,6 +522,35 @@ class CDB(object):
             }
             await f.write(dill.dumps(to_save))
 
+    def load_config(self, config_path: str) -> None:
+        if not os.path.exists(config_path):
+            if not self._config_from_file:
+                # if there's no config defined anywhere
+                raise ValueError("Could not find a config in the CDB nor "
+                                 "in the config.json for this model "
+                                 f"({os.path.dirname(config_path)})",
+                                 )
+            # if there is a config, but it's defined in the cdb.dat file
+            logger.warning("Could not find config.json in model pack folder "
+                           f"({os.path.dirname(config_path)}). "
+                           "This is probably an older model. Please save the model "
+                           "again in the new format to avoid potential issues.")
+        else:
+            if self._config_from_file:
+                # if there's a config.json and one defined in the cbd.dat file
+                raise ValueError("Found a config in the CDB and in the config.json "
+                                 f"for model ({os.path.dirname(config_path)}) - "
+                                 "this is ambiguous. Please either remove the "
+                                 "config.json or load the CDB without the config.json "
+                                 "in the folder and re-save in the newer format "
+                                 "(the default save in this version)")
+            # if the only config is in the separate config.json file
+            # this should be the behaviour for all newer models
+            self.config = cast(Config, Config.load(config_path))
+            logger.debug("Loaded config from CDB from %s", config_path)
+        # mark config read from file
+        self._config_from_file = True
+
     @classmethod
     def load(cls, path: str, json_path: Optional[str] = None, config_dict: Optional[Dict] = None) -> "CDB":
         """Load and return a CDB. This allows partial loads in probably not the right way at all.
@@ -457,6 +603,7 @@ class CDB(object):
 
                 # Increase the vector count
                 self.cui2count_train[cui] = self.cui2count_train.get(cui, 0) + cdb.cui2count_train[cui]
+        self.is_dirty = True
 
     def reset_cui_count(self, n: int = 10) -> None:
         """Reset the CUI count for all concepts that received training, used when starting new unsupervised training
@@ -472,6 +619,7 @@ class CDB(object):
         """
         for cui in self.cui2count_train.keys():
             self.cui2count_train[cui] = n
+        self.is_dirty = True
 
     def reset_training(self) -> None:
         """Will remove all training efforts - in other words all embeddings that are learnt
@@ -481,6 +629,28 @@ class CDB(object):
         self.cui2count_train = {}
         self.cui2context_vectors = {}
         self.reset_concept_similarity()
+        self.is_dirty = True
+
+    def populate_cui2snames(self, force: bool = True) -> None:
+        """Populate the cui2snames dict if it's empty.
+
+        If the dict is not empty and the population is not force,
+        nothing will happen.
+
+        For now, this method simply populates all the names form
+        cui2names into cui2snames.
+
+        Args:
+            force (bool, optional): Whether to force the (re-)population. Defaults to True.
+        """
+        if not force and self.cui2snames:
+            return
+        self.cui2snames.clear() # in case forced re-population
+        # run through cui2names
+        # and create new sets so that they can be independently modified
+        for cui, names in self.cui2names.items():
+            self.cui2snames[cui] = set(names)  # new set
+        self.is_dirty = True
 
     def filter_by_cui(self, cuis_to_keep: Union[List[str], Set[str]]) -> None:
         """Subset the core CDB fields (dictionaries/maps). Note that this will potenitally keep a bit more CUIs
@@ -489,6 +659,10 @@ class CDB(object):
         This also will not remove any data from cdb.addl_info - as this field can contain data of
         unknown structure.
 
+        As a side note, if the CDB has been memory-optimised, filtering will undo this memory optimisation.
+        This is because the dicts being involved will be rewritten.
+        However, the memory optimisation can be performed again afterwards.
+
         Args:
             cuis_to_keep (List[str]):
                 CUIs that will be kept, the rest will be removed (not completely, look above).
@@ -551,6 +725,9 @@ class CDB(object):
         self.cui2tags = new_cui2tags
         self.cui2type_ids = new_cui2type_ids
         self.cui2preferred_name = new_cui2preferred_name
+        self.is_dirty = True
+        # reset memory optimisation state
+        self._memory_optimised_parts.clear()
 
     def make_stats(self):
         stats = {}
@@ -570,6 +747,7 @@ class CDB(object):
     def reset_concept_similarity(self) -> None:
         """Reset concept similarity matrix."""
         self.addl_info['similarity'] = {}
+        self.is_dirty = True
 
     def most_similar(self,
                      cui: str,
@@ -692,13 +870,57 @@ which may or may not work. If you experience any compatibility issues, please re
 or download the compatible model."""
             )
 
-    def get_hash(self):
+    def _should_recalc_hash(self, force_recalc: bool) -> bool:
+        if force_recalc:
+            return True
+        if self.config.hash is None:
+            # TODO - perhaps this is not the best?
+            # as this is a side effect
+            # get and save result in config
+            self.config.get_hash()
+        if not self._hash or self.is_dirty:
+            # if no hash saved or is dirty
+            # need to calculate
+            logger.debug("Recalculating hash due to %s",
+                         "no hash saved" if not self._hash else "CDB is dirty")
+            return True
+        # recalc config hash in case it changed
+        self.config.get_hash()
+        if self._config_hash is None or self._config_hash != self.config.hash:
+            # if no config hash saved
+            # or if the config hash is different from one saved in here
+            logger.debug("Recalculating hash due to %s",
+                         "no config hash saved" if not self._config_hash
+                         else "config hash has changed")
+            return True
+        return False
+
+    def get_hash(self, force_recalc: bool = False):
+        should_recalc = self._should_recalc_hash(force_recalc)
+        if not should_recalc:
+            logger.info("Reusing old hash of CDB since the CDB has not changed: %s", self._hash)
+            return self._hash
+        self.is_dirty = False
+        return self.calculate_hash()
+
+    def calculate_hash(self):
+        logger.info("Recalculating hash for CDB, this may take a while")
         hasher = Hasher()
 
         for k,v in self.__dict__.items():
             if k in ['cui2countext_vectors', 'name2cuis']:
                 hasher.update(v, length=False)
+            elif k in ['_hash', 'is_dirty', '_config_hash']:
+                # ignore _hash since if it previously didn't exist, the
+                # new hash would be different when the value does exist
+                # and ignore is_dirty so that we get the same hash as previously
+                continue
             elif k != 'config':
                 hasher.update(v, length=True)
 
-        return hasher.hexdigest()
+        # set cached config hash
+        self._config_hash = self.config.hash
+
+        self._hash = hasher.hexdigest()
+        logger.info("Found new CDB hash: %s", self._hash)
+        return self._hash
