@@ -1,14 +1,12 @@
-from typing import Any, Dict, List, NewType
+from typing import Any, Dict, List
 import torch
 
-InputDataClass = NewType("InputDataClass", Any)
-
 
 class CollateAndPadNER(object):
     def __init__(self, pad_id):
         self.pad_id = pad_id
 
-    def __call__(self, features: List[InputDataClass]) -> Dict[str, torch.Tensor]:
+    def __call__(self, features: List[Any]) -> Dict[str, torch.Tensor]:
         batch = {}
 
         max_len = max([len(f['input_ids']) for f in features])
@@ -18,5 +16,4 @@ class CollateAndPadNER(object):
             max(0, max_len - len(f['labels'])) for f in features], dtype=torch.long)
         batch['attention_mask'] = batch['input_ids'] != self.pad_id
 
-
         return batch
