@@ -1,6 +1,7 @@
 import os
 import json
 import logging
+import datasets
 from spacy.tokens import Doc
 from datetime import datetime
 from typing import Iterable, Iterator, Optional, Dict, List, cast, Union
@@ -18,7 +19,7 @@ from medcat.datasets.data_collator import CollateAndPadNER
 
 from transformers import Trainer, AutoModelForTokenClassification, AutoTokenizer
 from transformers import pipeline, TrainingArguments
-import datasets
+from transformers.trainer_callback import TrainerCallback
 
 # It should be safe to do this always, as all other multiprocessing
 #will be finished before data comes to meta_cat
@@ -137,7 +138,12 @@ class TransformersNER(object):
 
         return out_path
 
-    def train(self, json_path: Union[str, list, None]=None, ignore_extra_labels=False, dataset=None, meta_requirements=None):
+    def train(self,
+              json_path: Union[str, list, None]=None,
+              ignore_extra_labels=False,
+              dataset=None,
+              meta_requirements=None,
+              trainer_callbacks: Optional[List[TrainerCallback]]=None):
         """Train or continue training a model give a json_path containing a MedCATtrainer export. It will
         continue training if an existing model is loaded or start new training if the model is blank/new.
 
@@ -149,6 +155,9 @@ class TransformersNER(object):
             ignore_extra_labels:
                 Makes only sense when an existing deid model was loaded and from the new data we want to ignore
                 labels that did not exist in the old model.
+            trainer_callbacks (List[TrainerCallback]):
+                A list of trainer callbacks for collecting metrics during the training at the client side. The
+                transformers Trainer object will be passed in when each callback is called.
         """
 
         if dataset is None and json_path is not None:
@@ -193,6 +202,9 @@ class TransformersNER(object):
                 compute_metrics=lambda p: metrics(p, tokenizer=self.tokenizer, dataset=encoded_dataset['test'], verbose=self.config.general['verbose_metrics']),
                 data_collator=data_collator, # type: ignore
                 tokenizer=None)
+        if trainer_callbacks:
+            for callback in trainer_callbacks:
+                trainer.add_callback(callback(trainer))
 
         trainer.train() # type: ignore
 
@@ -331,19 +343,19 @@ class TransformersNER(object):
             return stream
 
         batch_size_chars = self.config.general['pipe_batch_size_in_chars']
-        yield from self._process(stream, batch_size_chars)
+        yield from self._process(stream, batch_size_chars)  # type: ignore
 
     def _process(self,
                  stream: Iterable[Union[Doc, None]],
                  batch_size_chars: int) -> Iterator[Optional[Doc]]:
-        for docs in self.batch_generator(stream, batch_size_chars):
+        for docs in self.batch_generator(stream, batch_size_chars):  # type: ignore
             #all_text = [doc.text for doc in docs]
             #all_text_processed = self.tokenizer.encode_eval(all_text)
             # For now we will process the documents one by one, should be improved in the future to use batching
             for doc in docs:
                 try:
                     res = self.ner_pipe(doc.text, aggregation_strategy=self.config.general['ner_aggregation_strategy'])
-                    doc.ents = []
+                    doc.ents = []  # type: ignore
                     for r in res:
                         inds = []
                         for ind, word in enumerate(doc):
@@ -353,15 +365,15 @@ class TransformersNER(object):
                             # To not loop through everything
                             if end_char > r['end']:
                                 break
+                        if inds:
+                            entity = Span(doc, min(inds), max(inds) + 1, label=r['entity_group'])
+                            entity._.cui = r['entity_group']
+                            entity._.context_similarity = r['score']
+                            entity._.detected_name = r['word']
+                            entity._.id = len(doc._.ents)
+                            entity._.confidence = r['score']
 
-                        entity = Span(doc, min(inds), max(inds) + 1, label=r['entity_group'])
-                        entity._.cui = r['entity_group']
-                        entity._.context_similarity = r['score']
-                        entity._.detected_name = r['word']
-                        entity._.id = len(doc._.ents)
-                        entity._.confidence = r['score']
-
-                        doc._.ents.append(entity)
+                            doc._.ents.append(entity)
                     create_main_ann(self.cdb, doc)
                     if self.cdb.config.general['make_pretty_labels'] is not None:
                         make_pretty_labels(self.cdb, doc, LabelStyle[self.cdb.config.general['make_pretty_labels']])
