@@ -6,10 +6,10 @@ from medcat.cdb import CDB
 from medcat.config import Config
 
 
+logger = logging.getLogger(__name__)
+
+
 class NER(PipeRunner):
-    r'''
-    '''
-    log = logging.getLogger(__name__)
 
     # Custom pipeline component name
     name = 'cat_ner'
@@ -18,20 +18,20 @@ class NER(PipeRunner):
     def __init__(self, cdb: CDB, config: Config) -> None:
         self.config = config
         self.cdb = cdb
-        super().__init__(self.config.general['workers'])
+        super().__init__(self.config.general.workers)
 
     # Override
     def __call__(self, doc: Doc) -> Doc:
-        r''' Detect candidates for concepts - linker will then be able to do the rest. It adds `entities` to the
+        """Detect candidates for concepts - linker will then be able to do the rest. It adds `entities` to the
         doc._.ents and each entity can have the entitiy._.link_candidates - that the linker will resolve.
 
         Args:
-            doc (`spacy.tokens.Doc`):
+            doc (spacy.tokens.Doc):
                 Spacy document to be annotated with named entities.
         Return
-            doc (`spacy.tokens.Doc`):
+            doc (spacy.tokens.Doc):
                 Spacy document with detected entities.
-        '''
+        """
         # Just take the tokens we need
         _doc = [tkn for tkn in doc if not tkn._.to_skip]
         for i in range(len(_doc)):
@@ -44,7 +44,7 @@ class NER(PipeRunner):
             for name_version in name_versions:
                 if name_version in self.cdb.snames:
                     if name:
-                        name = name + self.config.general['separator'] + name_version
+                        name = name + self.config.general.separator + name_version
                     else:
                         name = name_version
                     break
@@ -53,7 +53,7 @@ class NER(PipeRunner):
 
             if name: # There has to be at least something appended to the name to go forward
                 for j in range(i+1, len(_doc)):
-                    if _doc[j].i - _doc[j-1].i - 1 > self.config.ner['max_skip_tokens']:
+                    if _doc[j].i - _doc[j-1].i - 1 > self.config.ner.max_skip_tokens:
                         # Do not allow to skip more than limit
                         break
                     tkn = _doc[j]
@@ -63,7 +63,7 @@ class NER(PipeRunner):
                     name_changed = False
                     name_reverse = None
                     for name_version in name_versions:
-                        _name = name + self.config.general['separator'] + name_version
+                        _name = name + self.config.general.separator + name_version
                         if _name in self.cdb.snames:
                             # Append the name and break
                             name = _name
@@ -71,7 +71,7 @@ class NER(PipeRunner):
                             break
 
                         if self.config.ner.get('try_reverse_word_order', False):
-                            _name_reverse = name_version + self.config.general['separator'] + name
+                            _name_reverse = name_version + self.config.general.separator + name
                             if _name_reverse in self.cdb.snames:
                                 # Append the name and break
                                 name_reverse = _name_reverse
