@@ -1,15 +1,23 @@
 import os
 from abc import ABC, abstractmethod
-from typing import List, Dict, Optional, Union
+from typing import List, Dict, Optional, Union, overload
 from tokenizers import Tokenizer, ByteLevelBPETokenizer
 from transformers.models.bert.tokenization_bert_fast import BertTokenizerFast
 
 
 class TokenizerWrapperBase(ABC):
 
+    name: str
+
     def __init__(self, hf_tokenizer: Optional[Tokenizer] = None) -> None:
         self.hf_tokenizers = hf_tokenizer
 
+    @overload
+    def __call__(self, text: str) -> Dict: ...
+
+    @overload
+    def __call__(self, text: List[str]) -> List[Dict]: ...
+
     @abstractmethod
     def __call__(self, text: Union[str, List[str]]) -> Union[Dict, List[Dict]]: ...
 
@@ -36,13 +44,13 @@ class TokenizerWrapperBase(ABC):
 
 
 class TokenizerWrapperBPE(TokenizerWrapperBase):
-    ''' Wrapper around a huggingface tokenizer so that it works with the
+    """Wrapper around a huggingface tokenizer so that it works with the
     MetaCAT models.
 
     Args:
-        hf_tokenizers (`tokenizers.ByteLevelBPETokenizer`):
+        tokenizers.ByteLevelBPETokenizer:
             A huggingface BBPE tokenizer.
-    '''
+    """
     name = 'bbpe'
 
     def __init__(self, hf_tokenizers: Optional[ByteLevelBPETokenizer] = None) -> None:
@@ -52,19 +60,24 @@ class TokenizerWrapperBPE(TokenizerWrapperBase):
             # For whatever reason added tokens do not persist with this tokenizer, what to do
             self.hf_tokenizers.add_tokens(['<PAD>'])
 
+    @overload
+    def __call__(self, text: str) -> Dict: ...
+
+    @overload
+    def __call__(self, text: List[str]) -> List[Dict]: ...
+
     def __call__(self, text: Union[str, List[str]]) -> Union[Dict, List[Dict]]:
-        r''' Tokenize some text
+        """Tokenize some text
 
         Args:
-            text (`Union(str, List[str])`):
+            text (Union(str, List[str])):
                 Text/texts to be tokenized.
 
         Returns:
-            res (`Union(dict, List[dict])`):
+            Union(dict, List[dict]):
                 Dictionary/ies containing `offset_mapping`, `input_ids` and `tokens` corresponding to the
                 input text/s.
-
-        '''
+        """
         self.hf_tokenizers = self.ensure_tokenizer()
 
         if isinstance(text, str):
@@ -123,18 +136,24 @@ class TokenizerWrapperBPE(TokenizerWrapperBase):
 
 
 class TokenizerWrapperBERT(TokenizerWrapperBase):
-    ''' Wrapper around a huggingface BERT tokenizer so that it works with the
+    """Wrapper around a huggingface BERT tokenizer so that it works with the
     MetaCAT models.
 
     Args:
-        hf_tokenizers (`transformers.models.bert.tokenization_bert_fast.BertTokenizerFast`):
+        transformers.models.bert.tokenization_bert_fast.BertTokenizerFast:
             A huggingface Fast BERT.
-    '''
+    """
     name = 'bert-tokenizer'
 
     def __init__(self, hf_tokenizers: Optional[BertTokenizerFast] = None) -> None:
         super().__init__(hf_tokenizers)
 
+    @overload
+    def __call__(self, text: str) -> Dict: ...
+
+    @overload
+    def __call__(self, text: List[str]) -> List[Dict]: ...
+
     def __call__(self, text: Union[str, List[str]]) -> Union[Dict, List[Dict]]:
         self.hf_tokenizers = self.ensure_tokenizer()
         if isinstance(text, str):
