@@ -9,37 +9,30 @@ from medcat.config_meta_cat import ConfigMetaCAT
 from medcat.tokenizers.meta_cat_tokenizers import TokenizerWrapperBERT
 
 
-class CATTests(unittest.TestCase):
+class MetaCATTests(unittest.TestCase):
 
     @classmethod
-    def setUpClass(self) -> None:
-        tokenizer = TokenizerWrapperBERT(AutoTokenizer.from_pretrained("bert-base-uncased"))
+    def setUpClass(cls) -> None:
+        tokenizer = TokenizerWrapperBERT(AutoTokenizer.from_pretrained('prajjwal1/bert-tiny'))
         config = ConfigMetaCAT()
         config.general['category_name'] = 'Status'
-        config.general['cntx_left'] = 15
-        config.general['cntx_right'] = 10
-        config.general['seed'] = 13
-        config.train['nepochs'] = 10
-        config.model['input_size'] = 300
-        config.train['auto_save_model'] = True
+        config.train['nepochs'] = 1
+        config.model['input_size'] = 100
 
-        self.meta_cat = MetaCAT(tokenizer=tokenizer, embeddings=None, config=config)
-
-        self.tmp_dir = os.path.join(os.path.dirname(os.path.realpath(__file__)), "tmp")
-        os.makedirs(self.tmp_dir, exist_ok=True)
+        cls.meta_cat = MetaCAT(tokenizer=tokenizer, embeddings=None, config=config)
 
+        cls.tmp_dir = os.path.join(os.path.dirname(os.path.realpath(__file__)), "tmp")
+        os.makedirs(cls.tmp_dir, exist_ok=True)
 
     def tearDown(self) -> None:
         shutil.rmtree(self.tmp_dir)
 
-
     def test_train(self):
         json_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'resources', 'mct_export_for_meta_cat_test.json')
         results = self.meta_cat.train(json_path, save_dir_path=self.tmp_dir)
 
         self.assertEqual(results['report']['weighted avg']['f1-score'], 1.0)
 
-
     def test_save_load(self):
         json_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'resources', 'mct_export_for_meta_cat_test.json')
         self.meta_cat.train(json_path, save_dir_path=self.tmp_dir)
