@@ -67,7 +67,8 @@ class A_NERTests(unittest.TestCase):
         cls.pipe.destroy()
 
     def test_aa_cdb_names_output(self):
-        target_result = {'S-229004': {'movar~viruse', 'movar', 'movar~viruses'}, 'S-229005': {'cdb'}}
+        print("Fixing 'movar~viruse' -> 'movar-virus' for newere en_core_web_md")
+        target_result = {'S-229004': {'movar~virus', 'movar', 'movar~viruses'}, 'S-229005': {'cdb'}}
         self.assertEqual(self.cdb.cui2names, target_result)
 
     def test_ab_entities_length(self):
