@@ -12,8 +12,19 @@ class DownloaderForm(forms.ModelForm):
         f" support team to report wider impact and usage of produced works"
         f" with the above information."
     ))
+
+    def __init__(self, models, *args, **kwargs):
+        super().__init__(*args, *kwargs)
+        self.fields["modelpack"] = forms.ChoiceField(label="Select a model for download",
+                                                     choices=[(
+                                                         model.model_name,
+                                                         f"{model.model_display_name}{' (' + model.model_description + ')' if model.model_description else ''}"
+                                                     ) for model in models],
+                                                     widget=forms.RadioSelect())
+
     class Meta:
         model = Downloader
+        exclude = ['downloaded_file']
         fields = [
             "first_name",
             "last_name",
